from datetime import datetime, timedelta
from dateutil import tz
from s3_backups.schedules import default
from s3_backups.postgres_to_s3 import archive
import unittest


class mockS3Key:

    def get_aws_datetime_string(self, dt):
        return "%s.000Z" % dt.strftime("%Y-%m-%dT%H:%M:%S")

    def __init__(self, utc_datetime):
        self.last_modified = self.get_aws_datetime_string(utc_datetime)
        self.name = "all_databases_%(year)s%(month)s%(day)s_%(hour)s%(minute)s%(second)s.tar.gz" % {
            'year': utc_datetime.year,
            'month': "%02d" % utc_datetime.month,
            'day': "%02d" % utc_datetime.day,
            'hour': "%02d" % utc_datetime.hour,
            'minute': "%02d" % utc_datetime.minute,
            'second': "%02d" % utc_datetime.second,
        }


class TestDefaultSchedule(unittest.TestCase):

    def setUp(self):
        self.utc = tz.tzutc()
        self.local_tz = tz.tzlocal()
        self.local_now = datetime.now()
        self.local_now = self.local_now.replace(tzinfo=self.local_tz)
        self.utcnow = datetime.utcnow()
        self.utcnow = self.utcnow.replace(tzinfo=self.utc)
        self.local_now_midnight = self.local_now.replace(hour=0, minute=0, second=0)
        self.utc_midnight = self.local_now_midnight.astimezone(self.utc)

    def get_key(self, dt):
        return archive.add_datetimes_to_key(mockS3Key(dt))

    def test_keep_all_archives_for_7_days(self):

        self.assertTrue(default.keep_file(self.get_key(self.utcnow - timedelta(days=6, hours=23, minutes=59, seconds=59))))
        self.assertTrue(default.keep_file(self.get_key(self.utcnow)))

    def test_gt_one_week_and_lt_one_month(self):

        default.oe.current = 'even'
        self.assertTrue(default.keep_file(self.get_key(self.utc_midnight - timedelta(days=9))))
        self.assertFalse(default.keep_file(self.get_key(self.utc_midnight - timedelta(days=10))))
        self.assertFalse(default.keep_file(self.get_key(self.utc_midnight - timedelta(days=10, hours=1))))

    def test_gt_one_month(self):

        # older than one month and not the first
        dt_1 = self.utc_midnight - timedelta(days=365)
        dt_1 = dt_1.replace(day=2)
        self.assertFalse(default.keep_file(self.get_key(dt_1)))

        # older than one month and the first
        dt_2 = dt_1.replace(day=1)
        self.assertTrue(default.keep_file(self.get_key(dt_2)))

if __name__ == '__main__':
    unittest.main()



from functools import wraps
import logging
import time

log = logging.getLogger('s3_backups')

# Define colors for the ColoredFormatter
BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)
COLORS = {
    'WARNING': CYAN,
    'INFO': WHITE,
    'DEBUG': GREEN,
    'CRITICAL': RED,
    'ERROR': RED
}
RESET_SEQ = "\033[0m"
COLOR_SEQ = "\033[1;%dm"
BOLD_SEQ = "\033[1m"


class ColoredFormatter(logging.Formatter):
    def __init__(self, msg, use_color=True):
        logging.Formatter.__init__(self, msg)
        self.use_color = use_color

    def format(self, record):
        levelname = record.levelname
        color = COLOR_SEQ % (30 + COLORS[levelname])
        message = logging.Formatter.format(self, record)
        message = message.replace("$RESET", RESET_SEQ).replace("$BOLD", BOLD_SEQ).replace("$COLOR", color)
        for k, v in COLORS.items():
            message = message.replace("$" + k, COLOR_SEQ % (v + 30)).replace("$BG" + k, COLOR_SEQ % (v + 40)).replace("$BG-" + k, COLOR_SEQ % (v + 40))
        return message + RESET_SEQ


def timeit(message="%(func_name)r (%(args)r, %(kwargs)r) %(time)s"):
    def _timeit(func):
        def _decorator(*args, **kwargs):
            ts = time.time()
            result = func(*args, **kwargs)
            te = time.time()
            elapsed = te - ts
            time_str = "%2.2f seconds" % elapsed
            if elapsed > 60:
                time_str = "%2.2f minutes" % (elapsed / 60)
            log.info(message % {'time': time_str, 'func_name': func.__name__, 'args': args, 'kwargs': kwargs})
            return result
        return wraps(func)(_decorator)
    return _timeit



#!/usr/bin/env python

import os
import sys
import s3_backups

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

setup(
    name='s3-backups',
    version=s3_backups.__version__,
    description='Backup stuff to S3',
    author="Brent O'Connor",
    author_email='epicserve@gmail.com',
    url='https://github.com/epicserve/s3-backups',
    packages=[
        's3_backups',
        's3_backups.schedules',
    ],
    package_data={'': ['LICENSE']},
    package_dir={'s3-backups': 's3-backups'},
    include_package_data=True,
    install_requires=['boto==2.9.4', 'python-dateutil==2.1'],
    license=open('LICENSE').read(),
    zip_safe=False,
    scripts=['s3_backups/postgres_to_s3.py', 's3_backups/redis_to_s3.py', 's3_backups/mysql_to_s3.py'],
    classifiers=(
        'Development Status :: 1 - Planning',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
    ),
)



#!/usr/bin/env python

from boto.s3.connection import S3Connection
from boto.s3.key import Key
from boto.exception import S3ResponseError
from datetime import datetime
from s3_backups.utils import ColoredFormatter, timeit
from dateutil import tz

import importlib
import tarfile
import subprocess
import tempfile
import argparse
import logging
import sys
import re

log = logging.getLogger('s3_backups')


@timeit("The backup took %(time)s to run")
def backup():
    """Backs up Redis to S3"""

    key_name = S3_KEY_NAME
    if not key_name.endswith("/") and key_name != "":
        key_name = "%s/" % key_name

    # add the file name date suffix
    now = datetime.now()
    FILENAME_SUFFIX = "_%(year)d%(month)02d%(day)02d_%(hour)02d%(minute)02d%(second)02d" % {
        'year': now.year,
        'month': now.month,
        'day': now.day,
        'hour': now.hour,
        'minute': now.minute,
        'second': now.second
    }
    FILENAME = ARCHIVE_NAME + FILENAME_SUFFIX + ".rdb.tar.gz"

    log.info("Preparing to dump the Redis database to " + FILENAME + " ...")

    # create postgres databeses dump
    proc1 = subprocess.Popen(REDIS_SAVE_CMD, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    proc1.wait()

    # create tar.gz for the above two files
    t2 = tempfile.NamedTemporaryFile()
    tar = tarfile.open(t2.name, "w|gz")
    tar.add(DUMP_RDB_PATH, ARCHIVE_NAME + ".rdb")
    tar.close()

    log.info("Uploading the " + FILENAME + " file to Amazon S3 ...")

    # get bucket
    conn = S3Connection(AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY)

    try:
        bucket = conn.get_bucket(S3_BUCKET_NAME)
    except S3ResponseError:
        sys.stderr.write("There is no bucket with the name \"" + S3_BUCKET_NAME + "\" in your Amazon S3 account\n")
        sys.stderr.write("Error: Please enter an appropriate bucket name and re-run the script\n")
        t2.close()
        return

    # upload file to Amazon S3
    k = Key(bucket)
    k.key = key_name + FILENAME
    k.set_contents_from_filename(t2.name)
    t2.close()

    log.info("Sucessfully uploaded the archive to Amazon S3")


class archive(object):
    """
    Archives all backups on S3 using the following schedule:

    - Keep all backups for 7 days
    - Keep midnight backups for every other day for 30 days
    - Keep 1st day of the month forever
    """

    def __init__(self, schedule_module='schedules.default'):

        schedule = importlib.import_module(schedule_module)
        conn = S3Connection(AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY)
        bucket = conn.get_bucket(S3_BUCKET_NAME)

        key_name = S3_KEY_NAME
        if not key_name.endswith("/") and key_name != "":
            key_name = "%s/" % key_name

        for key in bucket.list(key_name):
            if not key.name.endswith("/"):

                key = self.add_datetimes_to_key(key)

                # create a new key that puts the archive in a year/month sub
                # directory if it's not in a year/month sub directory already
                name_parts = key.name.split('/')
                month = name_parts[-2]
                year = name_parts[-3]
                new_key_name = key.name
                if not re.match(r'[\d]{4}', year) and not re.match(r'[\d]{2}', month):
                    name_parts.insert(len(name_parts) - 1, "%d" % key.local_last_modified.year)
                    name_parts.insert(len(name_parts) - 1, "%02d" % key.local_last_modified.month)
                    new_key_name = "/".join(name_parts)

                # either keep the file or delete it
                keep_file = schedule.keep_file(key)
                if keep_file and key.name != new_key_name:
                    key.copy(S3_BUCKET_NAME, new_key_name, metadata=key.metadata, preserve_acl=True)
                    bucket.delete_key(key.name)
                elif not keep_file:
                    bucket.delete_key(key.name)

    @classmethod
    def add_datetimes_to_key(self, key):
        """
        Convert the last_modified GMT datetime string to a datetime object and
        create utc and local datetime objects.
        """

        utc = tz.tzutc()
        gmt = tz.gettz('GMT')
        local_tz = tz.tzlocal()

        key.last_modified = datetime.strptime(key.last_modified, "%Y-%m-%dT%H:%M:%S.%fZ")
        key.last_modified = key.last_modified.replace(tzinfo=gmt)
        key.utc_last_modified = key.last_modified.astimezone(utc)
        key.local_last_modified = key.last_modified.astimezone(local_tz)

        return key


if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='Backs up Redis to S3 or archives backups.')

    # required arguments
    parser.add_argument('--AWS_ACCESS_KEY_ID', required=True, help='S3 access key')
    parser.add_argument('--AWS_SECRET_ACCESS_KEY', required=True, help='S3 secret access key')
    parser.add_argument('--S3_BUCKET_NAME', required=True, help='S3 bucket name')
    parser.add_argument('--S3_KEY_NAME', required=True, help='S3 key name, the directory path where you want to put archive (i.e. backups/redis/server_name)')

    # optional arguments
    parser.add_argument('-v', '--verbose', action='store_true', help='Verbose output')
    parser.add_argument('--DUMP_RDB_PATH', default='/var/lib/redis/dump.rdb', help="The path to the Redis dump.rdb file (default: /var/lib/redis/dump.rdb)")
    parser.add_argument('--REDIS_SAVE_CMD', default='/usr/bin/redis-cli SAVE', help="Command to save the Redis DB to disk (default: /usr/bin/redis-cli SAVE)")
    parser.add_argument('--ARCHIVE_NAME', default='dump', help='The base name for the archive')
    parser.add_argument('--schedule_module', default='s3_backups.schedules.default', help='Use a different archive schedule module (default: schedules.default)')
    parser.add_argument('--backup', action='store_true', help='Backup up Postgres to S3')
    parser.add_argument('--archive', action='store_true', help='Archive backups on S3')
    args = parser.parse_args()

    AWS_ACCESS_KEY_ID = args.AWS_ACCESS_KEY_ID
    AWS_SECRET_ACCESS_KEY = args.AWS_SECRET_ACCESS_KEY
    S3_BUCKET_NAME = args.S3_BUCKET_NAME
    S3_KEY_NAME = args.S3_KEY_NAME
    REDIS_SAVE_CMD = args.REDIS_SAVE_CMD
    ARCHIVE_NAME = args.ARCHIVE_NAME
    DUMP_RDB_PATH = args.DUMP_RDB_PATH

    if args.verbose:
        log.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        formatter = formatter = ColoredFormatter("$COLOR%(levelname)s: %(message)s$RESET")
        ch.setFormatter(formatter)
        log.addHandler(ch)

    if args.backup:
        backup()

    if args.archive:
        archive(args.schedule_module)
